from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.messages import Messages
from ..models.model import Model
from ..models.project import Project
from ..models.trace_feedback import TraceFeedback
from ..models.trace_retrieve_document_chunk import TraceRetrieveDocumentChunk
from ..types import UNSET, Unset

T = TypeVar("T", bound="TraceRetrieve")


class TraceRetrieveDict(TypedDict):
    trace_id: str
    project: "Project"
    model: "Model"
    feedback: Union["TraceFeedback", None]
    session_id: NotRequired[Union[None, Unset, str]]
    messages: NotRequired[Union[Unset, List["Messages"]]]
    document_chunks: NotRequired[Union[Unset, List["TraceRetrieveDocumentChunk"]]]
    pass


@_attrs_define
class TraceRetrieve:
    """
    Attributes:
        trace_id (str):
        project (Project):
        model (Model):
        feedback (Union['TraceFeedback', None]):
        session_id (Union[None, Unset, str]):
        messages (Union[Unset, List['Messages']]):
        document_chunks (Union[Unset, List['TraceRetrieveDocumentChunk']]):
    """

    trace_id: str
    project: "Project"
    model: "Model"
    feedback: Union["TraceFeedback", None]
    session_id: Union[None, Unset, str] = UNSET
    messages: Union[Unset, List["Messages"]] = UNSET
    document_chunks: Union[Unset, List["TraceRetrieveDocumentChunk"]] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.trace_feedback import TraceFeedback

        trace_id = self.trace_id

        project = self.project.to_dict()

        model = self.model.to_dict()

        feedback: Union[Dict[str, Any], None]
        if isinstance(self.feedback, TraceFeedback):
            feedback = self.feedback.to_dict()
        else:
            feedback = self.feedback

        session_id: Union[None, Unset, str]
        if isinstance(self.session_id, Unset):
            session_id = UNSET
        else:
            session_id = self.session_id

        messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.messages, Unset):
            messages = []
            for messages_item_data in self.messages:
                messages_item = messages_item_data.to_dict()
                messages.append(messages_item)

        document_chunks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.document_chunks, Unset):
            document_chunks = []
            for document_chunks_item_data in self.document_chunks:
                document_chunks_item = document_chunks_item_data.to_dict()
                document_chunks.append(document_chunks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "trace_id": trace_id,
                "project": project,
                "model": model,
                "feedback": feedback,
            }
        )
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if messages is not UNSET:
            field_dict["messages"] = messages
        if document_chunks is not UNSET:
            field_dict["document_chunks"] = document_chunks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.messages import Messages
        from ..models.model import Model
        from ..models.project import Project
        from ..models.trace_feedback import TraceFeedback
        from ..models.trace_retrieve_document_chunk import TraceRetrieveDocumentChunk

        d = src_dict.copy() if src_dict else {}
        trace_id = d.pop("trace_id")

        project = Project.from_dict(d.pop("project"))

        model = Model.from_dict(d.pop("model"))

        def _parse_feedback(data: object) -> Union["TraceFeedback", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                feedback_type_1 = TraceFeedback.from_dict(data)

                return feedback_type_1
            except:  # noqa: E722
                pass
            return cast(Union["TraceFeedback", None], data)

        feedback = _parse_feedback(d.pop("feedback"))

        def _parse_session_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        session_id = _parse_session_id(d.pop("session_id", UNSET))

        messages = []
        _messages = d.pop("messages", UNSET)
        for messages_item_data in _messages or []:
            messages_item = Messages.from_dict(messages_item_data)

            messages.append(messages_item)

        document_chunks = []
        _document_chunks = d.pop("document_chunks", UNSET)
        for document_chunks_item_data in _document_chunks or []:
            document_chunks_item = TraceRetrieveDocumentChunk.from_dict(document_chunks_item_data)

            document_chunks.append(document_chunks_item)

        trace_retrieve = cls(
            trace_id=trace_id,
            project=project,
            model=model,
            feedback=feedback,
            session_id=session_id,
            messages=messages,
            document_chunks=document_chunks,
        )

        trace_retrieve.additional_properties = d
        return trace_retrieve

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Dict, List, Optional

import httpx
from typing_extensions import Any

from ... import errors
from ...models.trace_list import TraceList

# from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response


def _get_kwargs(
    project_id: int,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["project_id"] = project_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/traces/",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client, response: httpx.Response) -> Optional[List["TraceList"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = TraceList.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client, response: httpx.Response) -> Response[List["TraceList"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_traces_list_wrapper(client):
    def v1_traces_list_wrapped(
        project_id: int,
    ) -> List["TraceList"]:
        """
        Args:
            project_id (int):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[List['TraceList']]
        """

        kwargs = _get_kwargs(
            project_id=project_id,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return v1_traces_list_wrapped



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.message import Message
from ..models.tool_call import ToolCall
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseChoice")


class ResponseChoiceDict(TypedDict):
    index: int
    message: "Message"
    finish_reason: str
    tool_calls: NotRequired[Union[List["ToolCall"], None, Unset]]
    pass


@_attrs_define
class ResponseChoice:
    """
    Attributes:
        index (int): The index of the choice in the list of choices.
        message (Message):
        finish_reason (str): The reason the chat completion finished, e.g., 'stop' or 'length'.
        tool_calls (Union[List['ToolCall'], None, Unset]): The tool calls made.
    """

    index: int
    message: "Message"
    finish_reason: str
    tool_calls: Union[List["ToolCall"], None, Unset] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        index = self.index

        message = self.message.to_dict()

        finish_reason = self.finish_reason

        tool_calls: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.tool_calls, Unset):
            tool_calls = UNSET
        elif isinstance(self.tool_calls, list):
            tool_calls = []
            for tool_calls_type_0_item_data in self.tool_calls:
                tool_calls_type_0_item = tool_calls_type_0_item_data.to_dict()
                tool_calls.append(tool_calls_type_0_item)

        else:
            tool_calls = self.tool_calls

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "index": index,
                "message": message,
                "finish_reason": finish_reason,
            }
        )
        if tool_calls is not UNSET:
            field_dict["tool_calls"] = tool_calls

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message import Message
        from ..models.tool_call import ToolCall

        d = src_dict.copy() if src_dict else {}
        index = d.pop("index")

        message = Message.from_dict(d.pop("message"))

        finish_reason = d.pop("finish_reason")

        def _parse_tool_calls(data: object) -> Union[List["ToolCall"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tool_calls_type_0 = []
                _tool_calls_type_0 = data
                for tool_calls_type_0_item_data in _tool_calls_type_0:
                    tool_calls_type_0_item = ToolCall.from_dict(tool_calls_type_0_item_data)

                    tool_calls_type_0.append(tool_calls_type_0_item)

                return tool_calls_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ToolCall"], None, Unset], data)

        tool_calls = _parse_tool_calls(d.pop("tool_calls", UNSET))

        response_choice = cls(
            index=index,
            message=message,
            finish_reason=finish_reason,
            tool_calls=tool_calls,
        )

        response_choice.additional_properties = d
        return response_choice

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.output_function_arguments import OutputFunctionArguments
from ..types import UNSET, Unset

T = TypeVar("T", bound="OutputFunction")


class OutputFunctionDict(TypedDict):
    name: str
    arguments: NotRequired[Union[Unset, OutputFunctionArguments]]
    pass


@_attrs_define
class OutputFunction:
    """
    Attributes:
        name (str): The name of the function to be called.
        arguments (Union[Unset, OutputFunctionArguments]): The arguments passed to the function.
    """

    name: str
    arguments: Union[Unset, "OutputFunctionArguments"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        arguments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.arguments, Unset):
            arguments = self.arguments.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if arguments is not UNSET:
            field_dict["arguments"] = arguments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.output_function_arguments import OutputFunctionArguments

        d = src_dict.copy() if src_dict else {}
        name = d.pop("name")

        _arguments = d.pop("arguments", UNSET)
        arguments: Union[Unset, OutputFunctionArguments]
        if isinstance(_arguments, Unset):
            arguments = UNSET
        else:
            arguments = OutputFunctionArguments.from_dict(_arguments)

        output_function = cls(
            name=name,
            arguments=arguments,
        )

        output_function.additional_properties = d
        return output_function

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Dict, List, Tuple, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.feedback_create_feedback import FeedbackCreateFeedback
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeedbackCreate")


class FeedbackCreateDict(TypedDict):
    trace_id: str
    feedback: NotRequired[Union["FeedbackCreateFeedback", None, Unset]]
    pass


@_attrs_define
class FeedbackCreate:
    """
    Attributes:
        trace_id (str):
        feedback (Union['FeedbackCreateFeedback', None, Unset]):
    """

    trace_id: str
    feedback: Union["FeedbackCreateFeedback", None, Unset] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.feedback_create_feedback import FeedbackCreateFeedback

        trace_id = self.trace_id

        feedback: Union[Dict[str, Any], None, Unset]
        if isinstance(self.feedback, Unset):
            feedback = UNSET
        elif isinstance(self.feedback, FeedbackCreateFeedback):
            feedback = self.feedback.to_dict()
        else:
            feedback = self.feedback

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "trace_id": trace_id,
            }
        )
        if feedback is not UNSET:
            field_dict["feedback"] = feedback

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        trace_id = (
            self.trace_id if isinstance(self.trace_id, Unset) else (None, str(self.trace_id).encode(), "text/plain")
        )

        feedback: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.feedback, Unset):
            feedback = UNSET
        elif isinstance(self.feedback, FeedbackCreateFeedback):
            feedback = (None, json.dumps(self.feedback.to_dict()).encode(), "application/json")
        else:
            feedback = self.feedback

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "trace_id": trace_id,
            }
        )
        if feedback is not UNSET:
            field_dict["feedback"] = feedback

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.feedback_create_feedback import FeedbackCreateFeedback

        d = src_dict.copy() if src_dict else {}
        trace_id = d.pop("trace_id")

        def _parse_feedback(data: object) -> Union["FeedbackCreateFeedback", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                feedback_type_1 = FeedbackCreateFeedback.from_dict(data)

                return feedback_type_1
            except:  # noqa: E722
                pass
            return cast(Union["FeedbackCreateFeedback", None, Unset], data)

        feedback = _parse_feedback(d.pop("feedback", UNSET))

        feedback_create = cls(
            trace_id=trace_id,
            feedback=feedback,
        )

        feedback_create.additional_properties = d
        return feedback_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.trace_feedback import TraceFeedback

T = TypeVar("T", bound="TraceList")


class TraceListDict(TypedDict):
    trace_id: str
    project_id: int
    model_id: int
    feedback: Union["TraceFeedback", None]
    pass


@_attrs_define
class TraceList:
    """
    Attributes:
        trace_id (str):
        project_id (int):
        model_id (int):
        feedback (Union['TraceFeedback', None]):
    """

    trace_id: str
    project_id: int
    model_id: int
    feedback: Union["TraceFeedback", None]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.trace_feedback import TraceFeedback

        trace_id = self.trace_id

        project_id = self.project_id

        model_id = self.model_id

        feedback: Union[Dict[str, Any], None]
        if isinstance(self.feedback, TraceFeedback):
            feedback = self.feedback.to_dict()
        else:
            feedback = self.feedback

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "trace_id": trace_id,
                "project_id": project_id,
                "model_id": model_id,
                "feedback": feedback,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trace_feedback import TraceFeedback

        d = src_dict.copy() if src_dict else {}
        trace_id = d.pop("trace_id")

        project_id = d.pop("project_id")

        model_id = d.pop("model_id")

        def _parse_feedback(data: object) -> Union["TraceFeedback", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                feedback_type_1 = TraceFeedback.from_dict(data)

                return feedback_type_1
            except:  # noqa: E722
                pass
            return cast(Union["TraceFeedback", None], data)

        feedback = _parse_feedback(d.pop("feedback"))

        trace_list = cls(
            trace_id=trace_id,
            project_id=project_id,
            model_id=model_id,
            feedback=feedback,
        )

        trace_list.additional_properties = d
        return trace_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Dict, Optional, Union, TypeVar, List, Type
from attrs import define as _attrs_define
from attrs import field as _attrs_field

import httpx
import json
from typing_extensions import Any, Unpack

from ... import errors
from ...models.api_response_validation_error import APIResponseValidationError
from ...models.authentication_error import AuthenticationError
from ...models.catch_all_error import CatchAllError
from ...models.chat_completion_input import ChatCompletionInput
from ...models.chat_completion_response import ChatCompletionResponse
from ...models.document_chunks import DocumentChunks
from ...models.conflict_error import ConflictError
from ...models.model_not_found_error import ModelNotFoundError
from ...models.permission_denied_error import PermissionDeniedError
from ...models.provider_api_connection_error import ProviderAPIConnectionError
from ...models.provider_api_status_error import ProviderAPIStatusError
from ...models.provider_api_timeout_error import ProviderAPITimeoutError
from ...models.provider_internal_server_error import ProviderInternalServerError
from ...models.provider_not_found_error import ProviderNotFoundError
from ...models.rate_limit_error import RateLimitError
from ...models.unprocessable_entity_error import UnprocessableEntityError
from ...models.validation_error import ValidationError
from ...models.usage import Usage
from ...models.response_choice import ResponseChoice
from ...types import Response, UNSET, Unset


# from ...client import AuthenticatedClient, Client
from ...types import Response
T = TypeVar("T", bound="ResponseChoice")

def _get_kwargs(
    **body: Unpack[ChatCompletionInput],
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/v1/chat/completions",
    }

    _json_body = body

    _kwargs["json"] = _json_body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client, response: httpx.Response
) -> Optional[
    Union[
        AuthenticationError,
        ChatCompletionResponse,
        ConflictError,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ChatCompletionResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client, response: httpx.Response
) -> Response[
    Union[
        AuthenticationError,
        ChatCompletionResponse,
        ConflictError,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_chat_completions_create_wrapper(client):
    def v1_chat_completions_create_wrapped(
        **body: Unpack[ChatCompletionInput],
    ) -> Union[
        AuthenticationError,
        ChatCompletionResponse,
        ConflictError,
        PermissionDeniedError,
        RateLimitError,
        Union[
            "APIResponseValidationError",
            "CatchAllError",
            "ProviderAPIConnectionError",
            "ProviderAPIStatusError",
            "ProviderAPITimeoutError",
            "ProviderInternalServerError",
        ],
        Union["ModelNotFoundError", "ProviderNotFoundError"],
        UnprocessableEntityError,
        ValidationError,
    ]:
        """Creates a model response for the given chat conversation.

        Args:
            authorization (str):
            body (ChatCompletionInput):
            body (ChatCompletionInput):
            body (ChatCompletionInput):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[Union[AuthenticationError, ChatCompletionResponse, ConflictError, PermissionDeniedError, RateLimitError, Union['APIResponseValidationError', 'CatchAllError', 'ProviderAPIConnectionError', 'ProviderAPIStatusError', 'ProviderAPITimeoutError', 'ProviderInternalServerError'], Union['ModelNotFoundError', 'ProviderNotFoundError'], UnprocessableEntityError, ValidationError]]
        """

        kwargs = _get_kwargs(
            **body,
        )

        httpx_client = client.get_httpx_client()
        if ('stream' in body) and (body['stream']):
            stream = httpx_client.stream(**kwargs)
            return ChatCompletionResponseStreamContainer(stream)
        else:
            response = httpx_client.request(
                **kwargs,
            )
            return _build_response(client=client, response=response).parsed
    return v1_chat_completions_create_wrapped


END_STREAM = 'done'
import re
class ChatCompletionResponseStreamContainer:
    _pattern = '(?<={key}: )(.*)'
    trace_id: str = None
    document_chunks: DocumentChunks = None

    def __init__(self, stream: httpx.Response):  
        self._stream = stream

    def _parse_data(self, text: str):
        try:
            match = re.search(self._pattern.format(key='data'), text)
            if match:
                data = json.loads(match.group(1))
                parsed_data: ChatCompletionResponseStream = ChatCompletionResponseStream.from_dict(data)
                return parsed_data
            else:   return None
        except Exception as e:
            return None

    def _parse_event(self, text: str):
        match = re.search(self._pattern.format(key='event'), text)
        if match: return match.group(1)
        else:   return None

    def _parse_trace_id(self, text: str):
        match = re.search(self._pattern.format(key='trace_id'), text)
        if match: return match.group(1)
        else:   return None

    def _parse_document_chunks(self, text: str):
        match = re.search(self._pattern.format(key='document_chunks'), text)
        if match:
            data = json.loads(match.group(1))
            parsed_data: DocumentChunks = [DocumentChunks.from_dict(parsed_chunk) for parsed_chunk in data]
            return parsed_data
        else:   return None
    

    def __iter__(self):
        def generator():
            with self._stream as stream:
                for texts in stream.iter_text():
                    
                    for text in texts.split('\n'):
                        trace_id = self._parse_trace_id(text)
                        self.trace_id = self.trace_id or trace_id
                        
                        self.document_chunks = self.document_chunks or self._parse_document_chunks(text)

                        data = self._parse_data(text)

                        if data:
                            yield data  
        return generator()
    

@_attrs_define
class ChatCompletionResponseStream:
    """
    Attributes:
        id (str): A unique identifier for the chat completion. Each chunk has the same ID.
        choices (List['ChatCompletionResponseChoicesItem']): A list of chat completion choices. Can be more
            than one if n is greater than 1.
        created (int): The Unix timestamp (in seconds) of when the chat completion was created. Each chunk has the same
            timestamp.
        model (str): The model to generate the completion.
        provider_name (str): The name of the provider that generated the completion.
        provider_id (str): The ID of the provider that generated the completion.
        usage (Union[Unset, ChatCompletionResponseUsage]): The usage statistics for the completion.
    """
    id: str
    choices: List["ResponseChoiceStream"]
    created: int
    model: str
    usage: Union[Unset, "Usage"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        choices = []
        for choices_item_data in self.choices:
            choices_item = choices_item_data.to_dict()
            choices.append(choices_item)

        created = self.created

        model = self.model

        usage: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.usage, Unset):
            usage = self.usage.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "choices": choices,
                "created": created,
                "model": model,
            }
        )
        if usage is not UNSET:
            field_dict["usage"] = usage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:

        d = src_dict.copy()
        id = d.pop("id")

        choices = []
        _choices = d.pop("choices")
        for choices_item_data in _choices:
            choices_item = ResponseChoiceStream.from_dict(choices_item_data)

            choices.append(choices_item)

        created = d.pop("created")
        model = d.pop("model")

        _usage = d.pop("usage", UNSET)
        usage: Union[Unset, Usage]
        if isinstance(_usage, Unset):
            usage = UNSET
        else:
            usage = Usage.from_dict(_usage)

        v1_chat_completions_create_response_200 = cls(
            id=id,
            choices=choices,
            created=created,
            model=model,
            usage=usage,
        )

        v1_chat_completions_create_response_200.additional_properties = d
        return v1_chat_completions_create_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


@_attrs_define
class ResponseChoiceStream:
    """
    Attributes:
        message (str): The generated message in the chat completion choice.
        finish_reason (str): The reason the chat completion finished, e.g., 'stop' or 'length'.
    """

    delta: str
    finish_reason: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        delta = self.delta

        finish_reason = self.finish_reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "delta": delta,
                "finish_reason": finish_reason,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        delta = d.pop("delta")

        finish_reason = d.pop("finish_reason")

        v1_chat_completions_create_response_200_choices_item = cls(
            delta=delta,
            finish_reason=finish_reason,
        )

        v1_chat_completions_create_response_200_choices_item.additional_properties = d
        return v1_chat_completions_create_response_200_choices_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


""" Contains methods for accessing the API """


from typing_extensions import Unpack

from ..models import (
    ChatCompletionInputDict,
    DocumentInputDict,
    EmbeddingsInputDict,
    FeedbackCreateDict,
    RepositoryDict,
)
from .chat_completions.v1_chat_completions_create import v1_chat_completions_create_wrapper
from .embeddings.v1_embeddings_create import v1_embeddings_create_wrapper
from .feedbacks.v1_set_trace_feedback_create import v1_set_trace_feedback_create_wrapper
from .models.v1_models_list import v1_models_list_wrapper
from .models.v1_models_retrieve import v1_models_retrieve_wrapper
from .repositories.api_repositories_repositories_create import api_repositories_repositories_create_wrapper
from .repository_document.api_repositories_repository_document_create import (
    api_repositories_repository_document_create_wrapper,
)
from .traces.v1_traces_list import v1_traces_list_wrapper
from .traces.v1_traces_retrieve import v1_traces_retrieve_wrapper
from .v1.v1_get_dataset_retrieve import v1_get_dataset_retrieve_wrapper


class RepositoriesModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[RepositoryDict]):
        return api_repositories_repositories_create_wrapper(self._client)(**kwargs)


class RepositoryDocumentModule:
    def __init__(self, client):
        self._client = client

    def create(self, repository_id: int, **kwargs: Unpack[DocumentInputDict]):
        return api_repositories_repository_document_create_wrapper(self._client)(repository_id, **kwargs)


class ChatCompletionsModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[ChatCompletionInputDict]):
        return v1_chat_completions_create_wrapper(self._client)(**kwargs)


class EmbeddingsModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[EmbeddingsInputDict]):
        return v1_embeddings_create_wrapper(self._client)(**kwargs)


class V1Module:
    def __init__(self, client):
        self._client = client

    def retrieve(
        self,
    ):
        return v1_get_dataset_retrieve_wrapper(self._client)()


class ModelsModule:
    def __init__(self, client):
        self._client = client

    def list(
        self,
    ):
        return v1_models_list_wrapper(self._client)()

    def retrieve(
        self,
        id: int,
    ):
        return v1_models_retrieve_wrapper(self._client)(
            id,
        )


class FeedbacksModule:
    def __init__(self, client):
        self._client = client

    def create(self, **kwargs: Unpack[FeedbackCreateDict]):
        return v1_set_trace_feedback_create_wrapper(self._client)(**kwargs)


class TracesModule:
    def __init__(self, client):
        self._client = client

    def list(
        self,
        project_id: int,
    ):
        return v1_traces_list_wrapper(self._client)(
            project_id,
        )

    def retrieve(
        self,
        id: str,
    ):
        return v1_traces_retrieve_wrapper(self._client)(
            id,
        )


class RepositoryModuleWrapper:
    document: RepositoryDocumentModule

    def __init__(self, client):
        self.document = RepositoryDocumentModule(client)


class ChatModuleWrapper:
    completions: ChatCompletionsModule

    def __init__(self, client):
        self.completions = ChatCompletionsModule(client)



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.blank_enum import BlankEnum
from ..models.model_provider_enum import ModelProviderEnum
from ..models.model_type_enum import ModelTypeEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="Model")


class ModelDict(TypedDict):
    id: int
    name: str
    model_type: NotRequired[Union[Unset, ModelTypeEnum]]
    model_provider: NotRequired[Union[BlankEnum, ModelProviderEnum, None, Unset]]
    deprecated: NotRequired[Union[Unset, bool]]
    pass


@_attrs_define
class Model:
    """
    Attributes:
        id (int):
        name (str):
        model_type (Union[Unset, ModelTypeEnum]): * `text2text` - Text to Text
            * `text2image` - Text to Image
            * `text2vector` - Text to Vector
        model_provider (Union[BlankEnum, ModelProviderEnum, None, Unset]):
        deprecated (Union[Unset, bool]):
    """

    id: int
    name: str
    model_type: Union[Unset, ModelTypeEnum] = UNSET
    model_provider: Union[BlankEnum, ModelProviderEnum, None, Unset] = UNSET
    deprecated: Union[Unset, bool] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        model_type: Union[Unset, str] = UNSET
        if not isinstance(self.model_type, Unset):
            model_type = self.model_type.value

        model_provider: Union[None, Unset, str]
        if isinstance(self.model_provider, Unset):
            model_provider = UNSET
        elif isinstance(self.model_provider, ModelProviderEnum):
            model_provider = self.model_provider.value
        elif isinstance(self.model_provider, BlankEnum):
            model_provider = self.model_provider.value
        else:
            model_provider = self.model_provider

        deprecated = self.deprecated

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if model_type is not UNSET:
            field_dict["model_type"] = model_type
        if model_provider is not UNSET:
            field_dict["model_provider"] = model_provider
        if deprecated is not UNSET:
            field_dict["deprecated"] = deprecated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        name = d.pop("name")

        _model_type = d.pop("model_type", UNSET)
        model_type: Union[Unset, ModelTypeEnum]
        if isinstance(_model_type, Unset):
            model_type = UNSET
        else:
            model_type = ModelTypeEnum(_model_type)

        def _parse_model_provider(data: object) -> Union[BlankEnum, ModelProviderEnum, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                model_provider_type_0 = ModelProviderEnum(data)

                return model_provider_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                model_provider_type_1 = BlankEnum(data)

                return model_provider_type_1
            except:  # noqa: E722
                pass
            return cast(Union[BlankEnum, ModelProviderEnum, None, Unset], data)

        model_provider = _parse_model_provider(d.pop("model_provider", UNSET))

        deprecated = d.pop("deprecated", UNSET)

        model = cls(
            id=id,
            name=name,
            model_type=model_type,
            model_provider=model_provider,
            deprecated=deprecated,
        )

        model.additional_properties = d
        return model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class APIResponseValidationErrorCodeEnum(str, Enum):
    APIRESPONSEVALIDATIONERROR = "APIResponseValidationError"

    def __str__(self) -> str:
        return str(self.value)



from .client import AuthenticatedClient

from .api import (
    RepositoriesModule,
    EmbeddingsModule,
    V1Module,
    ModelsModule,
    FeedbacksModule,
    TracesModule,
    RepositoryModuleWrapper,
    ChatModuleWrapper,
)

class Prem:
    repositories: RepositoriesModule
    embeddings: EmbeddingsModule
    v1: V1Module
    models: ModelsModule
    feedbacks: FeedbacksModule
    traces: TracesModule
    repository: RepositoryModuleWrapper
    chat: ChatModuleWrapper

    def __init__(self, api_key: str, base_url='https://app.premai.io'):
        client = AuthenticatedClient(token=api_key, base_url=base_url)
        # Init modules
        self.repositories = RepositoriesModule(client)
        self.embeddings = EmbeddingsModule(client)
        self.v1 = V1Module(client)
        self.models = ModelsModule(client)
        self.feedbacks = FeedbacksModule(client)
        self.traces = TracesModule(client)
        self.repository = RepositoryModuleWrapper(client)
        self.chat = ChatModuleWrapper(client)

__all__ = [
    "Prem"
]


from enum import Enum


class StatusEnum(str, Enum):
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    INDEXING = "INDEXING"
    PENDING = "PENDING"
    UPLOADED = "UPLOADED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.parameters import Parameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="Function")


class FunctionDict(TypedDict):
    name: str
    parameters: "Parameters"
    description: NotRequired[Union[Unset, str]]
    pass


@_attrs_define
class Function:
    """
    Attributes:
        name (str):
        parameters (Parameters):
        description (Union[Unset, str]):
    """

    name: str
    parameters: "Parameters"
    description: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        parameters = self.parameters.to_dict()

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "parameters": parameters,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.parameters import Parameters

        d = src_dict.copy() if src_dict else {}
        name = d.pop("name")

        parameters = Parameters.from_dict(d.pop("parameters"))

        description = d.pop("description", UNSET)

        function = cls(
            name=name,
            parameters=parameters,
            description=description,
        )

        function.additional_properties = d
        return function

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
